@@ -9,15 +9,17 @@
 #   scalper and dca bot
 #   unit-tests
 #   more docs and examples
-#   make normal logs
 #   decimal
 #   3.9
+#   subplot plot directory
 
 from copy import copy
 from datetime import datetime
 from threading import Thread
 from time import ctime, sleep, time
 from typing import Dict, List, Tuple, Any, Iterable, Union, Sized
+from re import fullmatch
+from warnings import warn
 
 import numpy as np
 import pandas as pd
@@ -36,29 +38,29 @@ Line = dict  # To avoid the deprecation warning
 
 class Trader(object):
     """
-    algo-trading system.
+    algo-trading main class.
     ticker:   |     str      |  ticker/symbol of chart
     df:       |   dataframe  |  data of chart
     interval: |     str      |  interval of df.
 
     """
-    _profit_calculate_coef: float
+    _profit_calculate_coef: Union[float, int]
     returns: utils.PREDICT_TYPE_LIST = []
     df: pd.DataFrame
     ticker: str
     interval: str
     __exit_order__: bool = False
     _prev_predict: str = 'Exit'
-    _stop_loss: float
-    _take_profit: float
+    _stop_loss: Union[float, int]
+    _take_profit: Union[float, int]
     _open_price: float
     trades: int = 0
     profits: int = 0
     losses: int = 0
     _stop_losses: List[float]
     _take_profits: List[float]
-    _credit_leverages: List[float]
-    deposit_history: List[float]
+    _credit_leverages: List[Union[float, int]]
+    deposit_history: List[Union[float, int]]
     year_profit: float
     average_growth: np.ndarray
     _info: str
@@ -72,15 +74,23 @@ class Trader(object):
     _sec_interval: int
     supports: Dict[int, float]
     resistances: Dict[int, float]
-    __last_credit_leverage: float
-    __prev_credit_lev: float = 1
+    __last_credit_leverage: Union[float, int]
+    __prev_credit_lev: Union[float, int] = 1
     trading_on_client: bool
 
+    @utils.assert_logger
     def __init__(self,
                  ticker: str = 'BTC/USDT',
                  df: pd.DataFrame = pd.DataFrame(),
                  interval: str = '1d',
                  trading_on_client: bool = True):
+        ticker = ticker.upper()
+        assert isinstance(ticker, str), 'Deposit can only be a number.'
+        assert fullmatch(r'[A-Z]+/[A-Z]+', ticker), 'Ticker must match the pattern <[A-Z]+/[A-Z]+>'
+        assert isinstance(df, pd.DataFrame), 'Dataframe can only be of type pd.DataFrame.'
+        assert isinstance(interval, str), 'interval can only be of the string type.'
+        assert isinstance(trading_on_client, bool), 'trading_on_client can only be True or False.'
+
         self.df = df.reset_index(drop=True)
         self.ticker = ticker
         self.interval = interval
@@ -136,13 +146,17 @@ class Trader(object):
         return {'stop': _stop_loss,
                 'take': take}
 
-    def sl_tp_adder(self, add_stop_loss: float = 0, add_take_profit: float = 0) -> Tuple[List[float], List[float]]:
+    @utils.assert_logger
+    def sl_tp_adder(self, add_stop_loss: Union[float, int] = 0.0, add_take_profit: Union[float, int] = 0.0) -> Tuple[List[float], List[float]]:
         """
 
         :param add_stop_loss: add stop loss points
         :param add_take_profit: add take profit points
         :return: (stop losses, take profits)
         """
+        assert isinstance(add_stop_loss, (int, float)) and isinstance(add_take_profit, (int, float)), \
+            'Arguments to this function can only be float or int.'
+
         utils.logger.debug('add stop-loss: %f pips, take-profit: %s pips', add_stop_loss, add_take_profit)
         stop_losses = []
         take_profits = []
@@ -292,10 +306,10 @@ class Trader(object):
         return self.returns
 
     def strategy_rsi(self,
-                     minimum: float = 20,
-                     maximum: float = 80,
-                     max_mid: float = 75,
-                     min_mid: float = 35,
+                     minimum: Union[float, int] = 20,
+                     maximum: Union[float, int] = 80,
+                     max_mid: Union[float, int] = 75,
+                     min_mid: Union[float, int] = 35,
                      **rsi_kwargs) -> utils.PREDICT_TYPE_LIST:
         self.returns = []
         rsi = ta.momentum.rsi(close=self.df['Close'], **rsi_kwargs)
@@ -452,7 +466,7 @@ class Trader(object):
                           kijunsen: int = 26,
                           senkouspan: int = 52,
                           chinkouspan: int = 26,
-                          stop_loss_plus: float = 40.0,
+                          stop_loss_plus: Union[float, int] = 40.0,  # sl_tp_adder
                           plot: bool = True) -> utils.PREDICT_TYPE_LIST:
         cloud = ta.trend.IchimokuIndicator(self.df["High"],
                                            self.df["Low"],
@@ -542,7 +556,11 @@ class Trader(object):
         self.sl_tp_adder(add_stop_loss=stop_loss_plus)
         return self.returns
 
+    @utils.assert_logger
     def crossover(self, fast: Iterable, slow: Iterable):
+        assert isinstance(fast, Iterable) and isinstance(slow, Iterable), \
+            'The arguments to this function must be iterable.'
+
         self.returns = []
         for s, f in zip(slow, fast):
             if s < f:
@@ -555,13 +573,19 @@ class Trader(object):
         self.set_open_stop_and_take()
         return self.returns
 
-    def inverse_strategy(self, swap_tpop_take: bool = True) -> utils.PREDICT_TYPE_LIST:
+    @utils.assert_logger
+    def inverse_strategy(self, swap_stop_take: bool = True) -> utils.PREDICT_TYPE_LIST:
         """
         makes signals inverse:
         buy = sell.
         sell = buy.
         exit = exit.
         """
+        assert isinstance(swap_stop_take, (bool, int, float)), 'swap_stop_take can only be boolean or int(float)'
+        if not isinstance(swap_stop_take, bool):
+            msg = 'swap_stop_take is not a standard type'
+            warn(msg)
+            utils.logger.info(msg)
 
         returns = []
         flag: utils.PREDICT_TYPE = utils.EXIT
@@ -574,14 +598,15 @@ class Trader(object):
                 flag = utils.EXIT
             returns.append(flag)
         self.returns = returns
-        if swap_tpop_take:
+        if swap_stop_take:
             self._stop_losses, self._take_profits = self._take_profits, self._stop_losses
         return self.returns
 
+    @utils.assert_logger
     def backtest(self,
-                 deposit: float = 10_000.0,
-                 bet: float = np.inf,
-                 commission: float = 0.0,
+                 deposit: Union[float, int] = 10_000.0,
+                 bet: Union[float, int] = np.inf,
+                 commission: Union[float, int] = 0.0,
                  plot: bool = True,
                  print_out: bool = True,
                  show: bool = True) -> pd.DataFrame:
@@ -595,6 +620,12 @@ class Trader(object):
         :param show: show the graph
         returns: pd.DataFrame with data of test
         """
+        assert isinstance(deposit, (float, int)), 'deposit must be of type int or float'
+        assert isinstance(bet, (float, int)), 'bet must be of type int or float'
+        assert isinstance(commission, (float, int)), 'commission must be of type int or float'
+        assert isinstance(plot, bool), 'plot must be of type bool'
+        assert isinstance(print_out, bool), 'print_out must be of type bool'
+        assert isinstance(show, bool), 'show must be of type bool'
 
         exit_take_stop: bool
         no_order: bool
@@ -605,9 +636,9 @@ class Trader(object):
         lin_calc_df: pd.DataFrame
         high: float
         low: float
-        credit_lev: float
+        credit_lev: Union[float, int]
 
-        start_bet: float = bet
+        start_bet: Union[float, int] = bet
         data_column: pd.Series = self.df['Close']
         data_high: pd.Series = self.df['High']
         data_low: pd.Series = self.df['Low']
@@ -616,8 +647,8 @@ class Trader(object):
         self.trades = 0
         self.profits = 0
         self.losses = 0
-        moneys_open_bet: float = deposit
-        money_start: float = deposit
+        moneys_open_bet: Union[float, int] = deposit
+        money_start: Union[float, int] = deposit
         oldsig = utils.EXIT
 
         e: int
@@ -820,17 +851,25 @@ winrate: {self.winrate}%"""
 
         return self.backtest_out
 
+    @utils.assert_logger
     def multi_backtest(self,
                        tickers: Union[Sized, Iterable[str]],
                        strategy_name: str = 'strategy_macd',
                        strategy_kwargs: Dict[str, Any] = {},
-                       deposit: float = 10_000.0,
-                       bet: float = np.inf,
-                       commission: float = 0.0,
+                       limit: int = 1000,
+                       deposit: Union[float, int] = 10_000.0,
+                       bet: Union[float, int] = np.inf,
+                       commission: Union[float, int] = 0.0,
                        plot: bool = True,
                        print_out: bool = True,
-                       show: bool = True,
-                       limit: int = 1000) -> pd.DataFrame:
+                       show: bool = True) -> pd.DataFrame:
+        assert isinstance(tickers, Iterable), 'tickers must be of type Iterable[str]'
+        for el in tickers:
+            assert isinstance(el, str), 'tickers must be of type Iterable[str]'
+        assert isinstance(strategy_name, str), 'strategy_name must be of type str'
+        assert strategy_name in self.__dir__(), 'There is no such strategy'
+        assert isinstance(strategy_kwargs, dict)
+
         winrates: List[float] = []
         percentage_profits: List[float] = []
         losses: List[int] = []
@@ -908,11 +947,12 @@ winrate: {self.winrate}%"""
             self.fig.show()
         return self.backtest_out
 
+    @utils.assert_logger
     def set_pyplot(self,
-                   height: int = 900,
-                   width: int = 1300,
+                   height: Union[int, float] = 900,
+                   width: Union[int, float] = 1300,
                    template: str = 'plotly_dark',
-                   row_heights: list = [10, 16, 7],
+                   row_heights: List[int, float] = [10, 16, 7],
                    **subplot_kwargs):
         """
 
@@ -921,6 +961,13 @@ winrate: {self.winrate}%"""
         :param template: plotly template
         :param row_heights: standard
         """
+        assert isinstance(height, (int, float)), 'height must be of type int or float'
+        assert isinstance(width, (int, float)), 'width must be of type int or float'
+        assert isinstance(template, str), 'template must be of type str'
+        assert isinstance(row_heights, list), 'row_heights must be of type List[int, float]'
+        for el in row_heights:
+            assert isinstance(el, (int, float))
+
         self.fig = make_subplots(3, 1, row_heights=row_heights, **subplot_kwargs)
         self.fig.update_layout(
             height=height,
@@ -928,13 +975,13 @@ winrate: {self.winrate}%"""
             template=template,
             xaxis_rangeslider_visible=False)
         self.fig.update_xaxes(
-            title_text='T I M E', row=3, col=1, color=utils.TEXT_COLOR)
+            title_text=utils.TIME_TITLE, row=3, col=1, color=utils.TEXT_COLOR)
         self.fig.update_yaxes(
-            title_text='M O N E Y S', row=2, col=1, color=utils.TEXT_COLOR)
+            title_text=utils.MONEYS_TITLE, row=2, col=1, color=utils.TEXT_COLOR)
         self.fig.update_yaxes(
-            title_text='R E T U R N S', row=3, col=1, color=utils.TEXT_COLOR)
+            title_text=utils.RETURNS_TITLE, row=3, col=1, color=utils.TEXT_COLOR)
         self.fig.update_yaxes(
-            title_text='D A T A', row=1, col=1, color=utils.TEXT_COLOR)
+            title_text=utils.DATA_TITLE, row=1, col=1, color=utils.TEXT_COLOR)
         utils.logger.info('new %s graph', self)
 
     def strategy_collider(self,
@@ -1057,7 +1104,7 @@ winrate: {self.winrate}%"""
         return self.returns
 
     def get_trading_predict(self,
-                            bet_for_trading_on_client: float = np.inf,
+                            bet_for_trading_on_client: Union[float, int] = np.inf,
                             coin_lotsize_division: bool = True
                             ) -> Dict[str, Union[str, float]]:
         """
@@ -1069,7 +1116,7 @@ winrate: {self.winrate}%"""
         """
 
         _moneys_: float
-        bet: float
+        bet: Union[float, int]
         close: np.ndarray = self.df["Close"].values
 
         # get prediction
@@ -1118,11 +1165,11 @@ winrate: {self.winrate}%"""
                          strategy,
                          ticker: str = 'BTC/USDT',
                          print_out: bool = True,
-                         bet_for_trading_on_client: float = np.inf,
+                         bet_for_trading_on_client: Union[float, int] = np.inf,
                          coin_lotsize_division: bool = True,
                          ignore_exceptions: bool = False,
                          print_exc: bool = True,
-                         wait_sl_tp_checking: float = 5,
+                         wait_sl_tp_checking: Union[float, int] = 5,
                          limit: int = 1000,
                          strategy_in_sleep: bool = False,
                          *strategy_args,
@@ -1199,21 +1246,21 @@ winrate: {self.winrate}%"""
                                start_time: datetime,  # LOCAL TIME
                                strategy_name: str,
                                print_out: bool = True,
-                               bet_for_trading_on_client: float = np.inf,  # for 1 trade
+                               bet_for_trading_on_client: Union[float, int] = np.inf,  # for 1 trade
                                coin_lotsize_division: bool = True,
                                ignore_exceptions: bool = False,
                                print_exc: bool = True,
-                               wait_sl_tp_checking: float = 5,
+                               wait_sl_tp_checking: Union[float, int] = 5,
                                limit: int = 1000,
                                strategy_in_sleep: bool = False,
-                               deposit_part: float = 1.0,  # for all trades
+                               deposit_part: Union[float, int] = 1.0,  # for all trades
                                **strategy_kwargs):
         can_orders: int = len(tickers)
-        bet_for_trading_on_client_copy: float = bet_for_trading_on_client
+        bet_for_trading_on_client_copy: Union[float, int] = bet_for_trading_on_client
 
         class MultiRealTimeTrader(self.__class__):
             def get_trading_predict(self,
-                                    bet_for_trading_on_client: float = np.inf,
+                                    bet_for_trading_on_client: Union[float, int] = np.inf,
                                     coin_lotsize_division: bool = True
                                     ) -> Dict[str, Union[str, float]]:
                 balance = self.client.get_balance_ticker(self.ticker.split('/')[1])
@@ -1280,8 +1327,8 @@ winrate: {self.winrate}%"""
         return self.returns
 
     def set_open_stop_and_take(self,
-                               take_profit: float = np.inf,
-                               stop_loss: float = np.inf,
+                               take_profit: Union[float, int] = np.inf,
+                               stop_loss: Union[float, int] = np.inf,
                                set_stop: bool = True,
                                set_take: bool = True):
         """
@@ -1320,7 +1367,7 @@ winrate: {self.winrate}%"""
                 self._stop_losses.append(stop_flag)
         utils.logger.debug('trader stop loss: %f pips, trader take profit: %f pips', stop_loss, take_profit)
 
-    def set_credit_leverages(self, credit_lev: float = 1.0):
+    def set_credit_leverages(self, credit_lev: Union[float, int] = 1.0):
         """
         Sets the leverage for bets.
         :param credit_lev: leverage in points
